/* 	Data Warehouse Script
	AdventureWorks Data Warehouse Example
	Author: Michael Sale - msale@stonehill.edu

	To see a screenshot of Microsoft Power BI dashboard, visit: https://i.imgur.com/58SOHnj.png
*/

/* Redirect Log File to Log Directory */
PROC PRINTTO log="\\server\folder\AWorksDWUpdate.log" NEW;
RUN;

/* Create SQL Server Libraries */
LIBNAME AW_PERS SQLSVR noprompt="uid=xxx; pwd=xxx; dsn=dsnname;" schema="person" stringdates=no;
LIBNAME AW_HR SQLSVR noprompt="uid=xxx; pwd=xxx; dsn=dsnname;" schema="humanresources" stringdates=no;
LIBNAME AW_PROD SQLSVR noprompt="uid=xxx; pwd=xxx; dsn=dsnname;" schema="production" stringdates=no;
LIBNAME AW_SALE SQLSVR noprompt="uid=xxx; pwd=xxx; dsn=dsnname;" schema="sales" stringdates=no;

/* Concatenate all SQL Server Libraries into AWORKS Library */
LIBNAME AWORKS (AW_PERS AW_HR AW_PROD AW_SALE);

/* Create Data Warehouse Library */
LIBNAME AWORKSDW BASE "\\server1\sas\data\AWorksDW";

/* Create Calendar Data Set */
%let start_date=01Jan1990;
%let end_date = %sysfunc(today(), date9.);

DATA WORK.DATE_BASE;
	DO DATE="&start_date"d TO "&end_date"d;
		OUTPUT;
	END;

	FORMAT DATE date9.;
RUN;

/* DIMENSION: Calendar */
PROC SQL;
	CREATE TABLE AWORKSDW.Date_Dimension AS

	SELECT 
		monotonic() AS DateKey,
		DATE format = DATETIME22.3 AS BaseDate,
		DATE format = mmddyy10. as Date,
		DATE format = YEAR. AS Year,
		DATE format = MONTH. AS Month,
		DATE format = MONNAME. AS MonthName,
		DATE format = EURDFDN. AS DayOfWeek,
		DATE format = DOWNAME. AS DayOfWeekName,
		DATE format = QTR. AS Quarter
	FROM	
		WORK.DATE_BASE;
RUN;

/* DIMENSION: Product */
PROC SQL;
	CREATE TABLE AWORKSDW.Product_Dimension AS

	SELECT
		monotonic() AS ProductKey,
		P.ProductID,
		P.Name AS ProductName,
		P.Size AS ProductSize,
		P.Style AS ProductStyle,
		P.Weight AS ProductWeight,
		P.ListPrice AS ProductListPrice,
		PSC.Name AS ProductSubCatName,
		PC.Name AS ProductCategoryName,
		PM.Name AS ProductModelName
	FROM 
		AWORKS.Product P
	LEFT JOIN AWORKS.ProductSubCategory AS PSC ON PSC.ProductSubcategoryID = P.ProductSubcategoryID
	LEFT JOIN AWORKS.ProductCategory AS PC ON PC.ProductCategoryID = PSC.ProductCategoryID
	LEFT JOIN AWORKS.ProductModel AS PM ON PM.ProductModelID = P.ProductModelID;
QUIT;

/* DIMENSION: Person */
PROC SQL;
	CREATE TABLE AWORKSDW.Person_Dimension AS

	SELECT
		monotonic() AS PersonKey,
		P.BusinessEntityID,
		P.FirstName,
		P.LastName,
		P.PersonType,
		E.EmailAddress,
		PH.PhoneNumber,
		PHT.Name AS PhoneNumberType
	FROM 
		AWORKS.Person AS P
	LEFT JOIN AWORKS.EmailAddress AS E ON E.BusinessEntityID = P.BusinessEntityID
	LEFT JOIN AWORKS.PersonPhone AS PH ON PH.BusinessEntityID = P.BusinessEntityID
	LEFT JOIN AWORKS.PhoneNumberType AS PHT ON PHT.PhoneNumberTypeID = PH.PhoneNumberTypeID;
QUIT;

/* DIMENSION: Order */
PROC SQL;
	CREATE TABLE AWORKSDW.Order_Dimension AS

	SELECT
		monotonic() AS OrderKey,
		S.SalesOrderID,
		S.ProductID,
		SH.OrderDate,
		SaT.Name AS TerritoryName,
		P.BusinessEntityID AS SalesPersonID
	FROM 
		AWORKS.SalesOrderDetail AS S
	LEFT JOIN AWORKS.SalesOrderHeader AS SH ON SH.SalesOrderID = S.SalesOrderID
	LEFT JOIN AWORKS.SalesTerritory AS SaT ON SaT.TerritoryID = SH.TerritoryID
	LEFT JOIN AWORKS.SalesPerson AS SP ON SP.BusinessEntityID = SH.SalesPersonID
	LEFT JOIN AWORKS.Person AS P ON P.BusinessEntityID = SP.BusinessEntityID;
QUIT;

/* FACT: Sales */
PROC SQL;
	CREATE TABLE AWORKSDW.Sales_Fact AS

	SELECT DISTINCT
		S_F.OrderQty,
		S_F.UnitPrice,
		S_F.OrderDate,
		O_D.OrderKey,
		PE_D.PersonKey,
		PR_D.ProductKey,
		D_D.DateKey
	FROM (
		SELECT
			OD.OrderKey,
			S_F.OrderQty,
			S_F.UnitPrice,
			SH.OrderDate,
			DATEPART(SH.OrderDate) format = mmddyy10. AS ShortOrderDate
		FROM
			AWORKS.SalesOrderDetail AS S_F
		LEFT JOIN AWORKS.SalesOrderHeader AS SH ON SH.SalesOrderID = S_F.SalesOrderID
		LEFT JOIN AWORKSDW.Order_Dimension AS OD ON OD.SalesOrderID = S_F.SalesOrderID AND OD.ProductID = S_F.ProductID
			) AS S_F
		LEFT JOIN AWORKSDW.Order_Dimension AS O_D ON O_D.OrderKey = S_F.OrderKey
		LEFT JOIN AWORKSDW.Person_Dimension AS PE_D ON PE_D.BusinessEntityID = O_D.SalesPersonID
		LEFT JOIN AWORKSDW.Product_Dimension AS PR_D ON PR_D.ProductID = O_D.ProductID
		LEFT JOIN AWORKSDW.Date_Dimension AS D_D ON D_D.Date = S_F.ShortOrderDate;
QUIT;

/* FACT: Cost History */
PROC SQL;
	CREATE TABLE AWORKSDW.CostHistory_Fact AS

	SELECT DISTINCT
		CH_F.ActualCost,
		CH_F.Quantity,
		CH_F.TransactionDate,
		CH_F.TransactionID,
		CH_F.ReferenceOrderID,
		OR_D.OrderKey,
		PR_D.ProductKey,
		D_D.DateKey
	FROM (
		SELECT
			CH_F.ActualCost,
			CH_F.Quantity,
			CH_F.ProductID,
			CH_F.TransactionID,
			CH_F.ReferenceOrderID,
			CH_F.TransactionDate,
			DATEPART(CH_F.TransactionDate) format = mmddyy10. AS ShortTrasDate
		FROM
			AWORKS.TransactionHistory AS CH_F
			) AS CH_F
		LEFT JOIN AWORKSDW.Product_Dimension AS PR_D ON PR_D.ProductID = CH_F.ProductID
		LEFT JOIN AWORKSDW.Order_Dimension AS OR_D ON OR_D.SalesOrderID = CH_F.ReferenceOrderID
		LEFT JOIN AWORKSDW.Date_Dimension AS D_D ON D_D.Date = CH_F.ShortTrasDate;
QUIT;

/* Employee List via Flat File Export from CRM */
PROC IMPORT datafile="\\server1\sas\import_files\employees.csv" 
	out=AWORKSDW.Employee_List 
	REPLACE;
RUN;

/* Departments List via Web Service API Call to HR System */
FILENAME resp temp;

PROC HTTP
	url="http://server.domain.com:80/rest/DemoApp/restService/HumanResources.Department/ALL"
	method="GET"
	out=resp;
RUN;

/* Use JSON LIBNAME engine to parse in response */
LIBNAME Depts JSON fileref=resp;

DATA AWORKSDW.Depts_List;
	SET Depts.data(rename=(element1=DepartmentID element2=DeptName element3=GroupName));
	DROP ordinal_root ordinal_DATA element4;
RUN;

/* Create Power BI Dashboard Data Set */
PROC SQL;
	CREATE TABLE AWORKSDW.POWER_BI AS
		SELECT
			SF.OrderQty,
			SF.UnitPrice,
			DD.Date,
			OD.TerritoryName,
			DD.Date AS OrderDate,
			PD.FirstName AS SalesPersonFirstName,
			PD.LastName AS SalesPersonLastName,
			PD.PersonType AS SalesPersonType,
			PrD.ProductName,
			PrD.ProductSubCatName,
			PrD.ProductCategoryName,
			PrD.ProductModelName
		FROM
			AWORKSDW.Sales_Fact AS SF
		LEFT JOIN AWORKSDW.Order_Dimension AS OD ON OD.OrderKey = SF.OrderKey
		LEFT JOIN AWORKSDW.Person_Dimension AS PD ON PD.PersonKey = SF.PersonKey
		LEFT JOIN AWORKSDW.Date_Dimension AS DD ON DD.DateKey = SF.DateKey
		LEFT JOIN AWORKSDW.Product_Dimension AS PrD ON PrD.ProductKey = SF.ProductKey;
QUIT;

/* Export Power BI Dashboard Data Set to CSV */
PROC EXPORT 
	dbms=csv
	data=AWORKSDW.POWER_BI 
	outfile="\\server1\sas\import_files\POWER_BI.csv"
	REPLACE;
RUN;

/* Send Email with Log */
/*
FILENAME sendit email 
    TO=("person@domain.com") 
    SUBJECT="LOG FILE: AdventureWorks Data Warehouse - Updated" 
    ATTACH=("\\server1\sas\saslogs\AWorksDWUpdate.log" 
            CONTENT_TYPE="text/plain");

DATA _null_;
    FILE sendit;
    PUT "Please find attached the log from the recent SAS program execution.";
RUN;
*/

/* Return Default Log Path */
PROC PRINTTO;
RUN;